@@ -2,18 +2,19 @@ import logging
 import re
 from datetime import datetime
 from itertools import groupby
+from decimal import Decimal
 
 from PySide6.QtWidgets import QApplication
 from jal.constants import PredefinedCategory
 from jal.widgets.helpers import ManipulateDate
-from jal.db.db import JalDB
-from jal.db.helpers import executeSQL, readSQLrecord
+from jal.db.account import JalAccount
+from jal.db.asset import JalAsset
 from jal.db.operations import Dividend
 from jal.data_import.statement import FOF, Statement_ImportError
 from jal.data_import.statement_xml import StatementXML
 
 JAL_STATEMENT_CLASS = "StatementIBKR"
-
+IBKR_CALCULATION_PRECISION = 10
 
 # -----------------------------------------------------------------------------------------------------------------------
 class IBKRCashOp:
@@ -37,7 +38,8 @@ class IBKR_AssetType:
         'BOND': FOF.ASSET_BOND,
         'OPT': FOF.ASSET_OPTION,
         'FUT': FOF.ASSET_FUTURES,
-        'WAR': FOF.ASSET_WARRANT
+        'WAR': FOF.ASSET_WARRANT,
+        'RIGHT': FOF.ASSET_RIGHTS
     }
 
     def __init__(self, asset_type, subtype):
@@ -62,6 +64,7 @@ class IBKR_CorpActionType:
         'FS': FOF.ACTION_SPLIT,            # Forward split
         'HI': FOF.PAYMENT_STOCK_DIVIDEND,  # Choice dividend
         'IC': FOF.ACTION_SYMBOL_CHANGE,    # Issue change
+        'RI': FOF.ACTION_RIGHTS_ISSUE,     # Subscribable Rights Issue
         'RS': FOF.ACTION_SPLIT,            # Reverse split
         'SO': FOF.ACTION_SPINOFF,          # Spin-off of new company
         'SD': FOF.PAYMENT_STOCK_DIVIDEND,  # Dividend paid in stocks
@@ -101,7 +104,8 @@ class IBKR_Account:
             else:
                 new_id = max([0] + [x['id'] for x in accounts_list]) + 1
                 account_ids.append(new_id)
-                account = {"id": new_id, "number": number, "currency": currency}
+                account = {"id": new_id, "number": number,
+                           "currency": currency, "precision": IBKR_CALCULATION_PRECISION}
                 accounts_list.append(account)
         if account_ids:
             if len(account_ids) == 1:
@@ -115,6 +119,7 @@ class IBKR_Account:
 class StatementIBKR(StatementXML):
     statements_path = '/*/FlexStatement'
     statement_tag = 'FlexStatement'
+    level_tag = 'levelOfDetail'
     CancelledFlag = 'Ca'
 
     def __init__(self):
@@ -144,7 +149,7 @@ class StatementIBKR(StatementXML):
                            'values': [('accountId', 'number', str, None),
                                       ('currency', 'currency', IBKR_Currency, None),
                                       ('startingCash', 'cash_begin', float, None),
-                                      ('endingCash', 'cash_end', float, None),    # -- this is planned
+                                      ('endingCash', 'cash_end', float, None),                   # -- this is planned
                                       ('endingSettledCash', 'cash_end_settled', float, None)],   # -- this is now
                            'loader': self.load_accounts},
             'SecuritiesInfo': {'tag': 'SecurityInfo',
@@ -213,6 +218,15 @@ class StatementIBKR(StatementXML):
                                             ('tradeID', 'number', str, ''),
                                             ('description', 'description', str, None)],
                                  'loader': self.load_cash_transactions},
+            'StockGrantActivities': {'tag': 'StockGrantActivity',
+                                     'level': '',
+                                     'values': [('accountId', 'account', IBKR_Account, None),
+                                                ('symbol', 'asset', IBKR_Asset, None),
+                                                ('awardDate', 'timestamp', datetime, None),
+                                                ('activityDescription', 'description', str, None),
+                                                ('quantity', 'amount', float, None),
+                                                ('price', 'price', float, None)],
+                                     'loader': self.load_vestings},
             'TransactionTaxes': {'tag': 'TransactionTax',
                                  'level': 'SUMMARY',
                                  'values': [('accountId', 'account', IBKR_Account, None),
@@ -345,6 +359,7 @@ class StatementIBKR(StatementXML):
     def load_accounts(self, balances):
         for i, balance in enumerate(sorted(balances, key=lambda x: x['currency'])):
             balance['id'] = i + 1
+            balance['precision'] = IBKR_CALCULATION_PRECISION
             self._data[FOF.ACCOUNTS].append(balance)
 
     def load_assets(self, assets):
@@ -447,7 +462,8 @@ class StatementIBKR(StatementXML):
             FOF.PAYMENT_STOCK_DIVIDEND: self.load_stock_dividend,
             FOF.ACTION_SPLIT: self.load_split,
             FOF.ACTION_BOND_MATURITY: self.load_bond_maturity,
-            FOF.ACTION_DELISTING: self.load_delisting
+            FOF.ACTION_DELISTING: self.load_delisting,
+            FOF.ACTION_RIGHTS_ISSUE: self.load_none
         }
 
         cnt = 0
@@ -466,7 +482,8 @@ class StatementIBKR(StatementXML):
             actions_aggregated.append(part)
             cnt += len(group_list) - 1
         # Now split in 2 parts: A for new stocks deposit, B for old stocks withdrawal
-        parts_a = [action for action in actions_aggregated if action['quantity'] >= 0]
+        # There might be 0 quantity value - it should be ignored
+        parts_a = [action for action in actions_aggregated if action['quantity'] > 0]
         parts_b = [action for action in actions_aggregated if action['quantity'] < 0]
         # Process sequentially '+' and '-', 'jal_processed' will set True when '+' has pair record in '-'
         for action in parts_a + parts_b:
@@ -509,13 +526,18 @@ class StatementIBKR(StatementXML):
             if action['code'] == StatementIBKR.CancelledFlag:
                 raise Statement_ImportError(self.tr("Can't process cancelled corporate action") + f" '{action}'")
 
+    # Dummy loader to skip some corporate actions
+    def load_none(self, action, parts_b) -> int:
+        return 0
+
     def load_merger(self, action, parts_b) -> int:
         MergerPatterns = [
             r"^(?P<symbol_old>.*)(.OLD)?\((?P<isin_old>\w+)\) +MERGED\([\w ]+\) +WITH +(?P<isin_new>\w+) +(?P<X>\d+) +FOR +(?P<Y>\d+) +\((?P<symbol>.*)(.OLD)?, (?P<name>.*), (?P<id>\w+)\)$",
-            r"^(?P<symbol_old>.*)(.OLD)?\((?P<isin_old>\w+)\) +CASH and STOCK MERGER +\([\w ]+\) +(?P<isin_new>\w+) +(?P<X>\d+) +FOR +(?P<Y>\d+) +AND +(?P<currency>.*) +(\d+(\.\d+)?) +\((?P<symbol>\w+)(.OLD)?, (?P<name>.*), (?P<id>\w+)\)$",
-            r"^(?P<symbol_old>.*)(.OLD)?\((?P<isin_old>\w+)\) +CASH and STOCK MERGER +\([\w ]+\) +(?P<isin_new>\w+) +(?P<X>\d+) +FOR +(?P<Y>\d+), +(?P<isin_new2>.*) +(?P<X2>\d+) +FOR +(?P<Y2>\d+) +AND +(?P<currency>\w+) +(\d+(\.\d+)?) +\((?P<symbol>\w+)(.OLD)?, (?P<name>.*), (?P<id>\w+)\)$",
+            r"^(?P<symbol_old>.*)(.OLD)?\((?P<isin_old>\w+)\) +CASH and STOCK MERGER +\([\w ]+\) +(?P<isin_new>[\w ]+) +(?P<X>\d+) +FOR +(?P<Y>\d+) +AND +(?P<currency>\w+) +(\d+(\.\d+)?) +\((?P<symbol>[\w ]+)(.OLD)?, (?P<name>.*), (?P<id>\w+)\)$",
+            r"^(?P<symbol_old>.*)(.OLD)?\((?P<isin_old>\w+)\) +CASH and STOCK MERGER +\([\w ]+\) +(?P<isin_new>[\w ]+) +(?P<X>\d+) +FOR +(?P<Y>\d+), +(?P<isin_new2>[\w ]+) +(?P<X2>\d+) +FOR +(?P<Y2>\d+) +AND +(?P<currency>\w+) +(\d+(\.\d+)?) +\((?P<symbol>[\w ]+)(.OLD)?, (?P<name>.*), (?P<id>\w+)\)$",
             r"^(?P<symbol_old>.*)\((?P<isin_old>\w+)\) +TENDERED TO +(?P<isin_new>\w+) +(?P<X>\d+) +FOR +(?P<Y>\d+) +\((?P<symbol>.*), +(?P<name>.*), +(?P<id>.*)\)$",
-            r"^(?P<symbol_old>.*)\((?P<isin_old>\w+)\) +MERGED\([\w ]+\) +FOR (?P<currency>\w+) (?P<price>\d+\.\d+) PER SHARE +\((?P<symbol>.*), (?P<name>.*)( - TENDER ODD LOT)?, (?P<id>\w+)\)$"  # "TENDER ODD LOT" part is optional
+            r"^(?P<symbol_old>.*)\((?P<isin_old>\w+)\) +MERGED\([\w ]+\) +FOR (?P<currency>\w+) (?P<price>\d+\.\d+) PER SHARE +\((?P<symbol>.*), (?P<name>.*)( - TENDER ODD LOT)?, (?P<id>\w+)\)$",  # "TENDER ODD LOT" part is optional
+            r"^(?P<symbol_old>.*)(.OLD)?\((?P<isin_old>\w+)\) +MERGED\([\w ]+\) +WITH +(?P<isin_new>.*) +(?P<X>\d+) +FOR +(?P<Y>\d+), +(?P<isin_new2>.*) +(?P<X2>\d+) +FOR +(?P<Y2>\d+) +\((?P<symbol>.*)(.OLD)?, (?P<name>.*), (?P<id>\w+)\)$"
             ]
 
         parts = None
@@ -535,7 +557,7 @@ class StatementIBKR(StatementXML):
         description_b = action['description'][:parts.span('symbol')[0]] + merger_a['symbol_old']
         asset_b = self.locate_asset(merger_a['symbol_old'], merger_a['isin_old'])
 
-        if pattern_id == 4:  # Asset converted to money -> we store it as a sell trade
+        if pattern_id == 4:  # Asset converted to money -> store it as a sell trade
             action['id'] = max([0] + [x['id'] for x in self._data[FOF.TRADES]]) + 1
             action['settlement'] = action['timestamp']
             action['price'] = action['proceeds'] / (-action['quantity'])
@@ -546,36 +568,40 @@ class StatementIBKR(StatementXML):
             return 1
 
         paired_record = self.find_corp_action_pair(asset_b, description_b, action, parts_b)
-        if (pattern_id == 1 or pattern_id == 2) and (not paired_record[0]['jal_processed']):
-            self.add_merger_payment(action['timestamp'], action['account'], paired_record[0]['proceeds'],
-                                    parts['currency'], action['description'])
-        if pattern_id == 2 and (not paired_record[0]['jal_processed']):
-            action['timestamp'] -= 1
-            action['type'] = FOF.ACTION_SPINOFF   # FIXME it is temporary workaround to keep 2 outgoing assets - one as from spin-off, another from merger
-        action['id'] = max([0] + [x['id'] for x in self._data[FOF.CORP_ACTIONS]]) + 1
-        action['cost_basis'] = 1.0
-        action['asset'] = [paired_record[0]['asset'], action['asset']]
-        if action['asset_type'] == FOF.ASSET_BOND:  # Adjust number of bonds
-            action['quantity'] = [-paired_record[0]['quantity']/IBKR_Asset.BondPrincipal,
-                                  action['quantity']/IBKR_Asset.BondPrincipal]
+        # Adjust quantity for bonds
+        adj_factor = IBKR_Asset.BondPrincipal if action['asset_type'] == FOF.ASSET_BOND else 1.0
+        existing_action = None
+        # Special processing if 1 asset is converted into two other assets
+        if pattern_id == 2 or pattern_id == 5:
+            existing_action = self.locate_existing_merger(action['timestamp'],
+                                                          action['account'], paired_record[0]['asset'])
+        if existing_action is None:
+            action['id'] = max([0] + [x['id'] for x in self._data[FOF.CORP_ACTIONS]]) + 1
+            action['outcome'] = [{'asset': action['asset'], 'quantity': action['quantity']/adj_factor, 'share': 0.0}]
+            action['asset'] = paired_record[0]['asset']
+            action['quantity'] = -paired_record[0]['quantity']/adj_factor
+            # Process cash payment if it is present as part of corporate action
+            if pattern_id == 1 or pattern_id == 2:
+                payment = {'asset': self.currency_id(parts['currency']),
+                           'quantity': paired_record[0]['proceeds'], 'share': 0.0}
+                action['outcome'].insert(0, payment)
+            self.drop_extra_fields(action, ["value", "proceeds", "code", "asset_type", "jal_processed"])
+            self._data[FOF.CORP_ACTIONS].append(action)
         else:
-            action['quantity'] = [-paired_record[0]['quantity'], action['quantity']]
-        self.drop_extra_fields(action, ["value", "proceeds", "code", "asset_type", "jal_processed"])
-        self._data[FOF.CORP_ACTIONS].append(action)
-
+            next_outcome = {'asset': action['asset'], 'quantity': action['quantity']/adj_factor, 'share': 0.0}
+            existing_action['outcome'].append(next_outcome)
         paired_record[0]['jal_processed'] = True
-
         return 2
 
-    def add_merger_payment(self, timestamp, account_id, amount, currency, description):
-        currency_id = self.currency_id(currency)
-        account = [x for x in self._data[FOF.ACCOUNTS] if x['id'] == account_id][0]
-        if account['currency'] != currency_id:
-            account_id = IBKR_Account(self._data[FOF.ACCOUNTS], account['number'], [currency_id]).id
-        payment_base = max([0] + [x['id'] for x in self._data[FOF.INCOME_SPENDING]]) + 1
-        payment = {'id': payment_base, 'account': account_id, 'timestamp': timestamp, 'peer': 0,
-                   'lines': [{'amount': amount, 'category': -PredefinedCategory.Interest, 'description': description}]}
-        self._data[FOF.INCOME_SPENDING].append(payment)
+    def locate_existing_merger(self, timestamp, account, asset):
+        existing_merger = list(filter(
+            lambda merger: merger['type'] == FOF.ACTION_MERGER and merger['timestamp'] == timestamp
+                           and merger['account'] == account and merger['asset'] == asset, self._data[FOF.CORP_ACTIONS]))
+        if len(existing_merger) == 0:
+            return None
+        if len(existing_merger) != 1:
+            raise Statement_ImportError(self.tr("Multiple merger records already exist at ") + f"{timestamp}")
+        return existing_merger[0]
 
     def load_spinoff(self, action, _parts_b) -> int:
         SpinOffPattern = r"^(?P<symbol_old>.*)\((?P<isin_old>\w+)\) +SPINOFF +(?P<X>\d+) +FOR +(?P<Y>\d+) +\((?P<symbol>.*), (?P<name>.*), (?P<id>\w+)\)$"
@@ -594,9 +620,10 @@ class StatementIBKR(StatementXML):
             raise Statement_ImportError(self.tr("Spin-off rounding error is too big ") + f"'{action}'")
         qty_old = round(qty_old)
         action['id'] = max([0] + [x['id'] for x in self._data[FOF.CORP_ACTIONS]]) + 1
-        action['cost_basis'] = 0.0
-        action['asset'] = [asset_old, action['asset']]
-        action['quantity'] = [qty_old, action['quantity']]
+        action['outcome'] = [{'asset': asset_old, 'quantity': qty_old, 'share': 0.0},
+                             {'asset': action['asset'], 'quantity': action['quantity'], 'share': 0.0}]
+        action['asset'] = asset_old
+        action['quantity'] = qty_old
         self.drop_extra_fields(action, ["value", "proceeds", "code", "asset_type", "jal_processed"])
         self._data[FOF.CORP_ACTIONS].append(action)
         return 1
@@ -614,9 +641,9 @@ class StatementIBKR(StatementXML):
         asset_b = self.locate_asset(isin_change['symbol_old'], isin_change['isin_old'])
         paired_record = self.find_corp_action_pair(asset_b, description_b, action, parts_b)
         action['id'] = max([0] + [x['id'] for x in self._data[FOF.CORP_ACTIONS]]) + 1
-        action['cost_basis'] = 1.0
-        action['asset'] = [paired_record[0]['asset'], action['asset']]
-        action['quantity'] = [-paired_record[0]['quantity'], action['quantity']]
+        action['outcome'] = [{'asset': action['asset'], 'quantity': action['quantity'], 'share': 1.0}]
+        action['asset'] = paired_record[0]['asset']
+        action['quantity'] = -paired_record[0]['quantity']
         self.drop_extra_fields(action, ["value", "proceeds", "code", "asset_type", "jal_processed"])
         self._data[FOF.CORP_ACTIONS].append(action)
         paired_record[0]['jal_processed'] = True
@@ -652,9 +679,8 @@ class StatementIBKR(StatementXML):
             qty_old = qty_delta / (int(split['X']) / int(split['Y']) - 1)
             qty_new = qty_old + qty_delta
             action['id'] = max([0] + [x['id'] for x in self._data[FOF.CORP_ACTIONS]]) + 1
-            action['cost_basis'] = 1.0
-            action['asset'] = [action['asset'], action['asset']]
-            action['quantity'] = [qty_old, qty_new]
+            action['outcome'] = [{'asset': action['asset'], 'quantity': qty_new, 'share': 1.0}]
+            action['quantity'] = qty_old
             self.drop_extra_fields(action, ["value", "proceeds", "code", "asset_type", "jal_processed"])
             self._data[FOF.CORP_ACTIONS].append(action)
             return 1
@@ -663,9 +689,9 @@ class StatementIBKR(StatementXML):
             asset_b = self.locate_asset(split['symbol_old'], split['isin_old'])
             paired_record = self.find_corp_action_pair(asset_b, description_b, action, parts_b)
             action['id'] = max([0] + [x['id'] for x in self._data[FOF.CORP_ACTIONS]]) + 1
-            action['cost_basis'] = 1.0
-            action['asset'] = [paired_record[0]['asset'], action['asset']]
-            action['quantity'] = [-paired_record[0]['quantity'], action['quantity']]
+            action['outcome'] = [{'asset': action['asset'], 'quantity': action['quantity'], 'share': 1.0}]
+            action['asset'] = paired_record[0]['asset']
+            action['quantity'] = -paired_record[0]['quantity']
             self.drop_extra_fields(action, ["value", "proceeds", "code", "asset_type", "jal_processed"])
             self._data[FOF.CORP_ACTIONS].append(action)
             paired_record[0]['jal_processed'] = True
@@ -685,17 +711,30 @@ class StatementIBKR(StatementXML):
         return 1
 
     def load_delisting(self, action, parts_b) -> int:
+        # There might be delisting for issued rights - we don't need to store it as it isn't a real asset
+        asset = [x for x in self._data[FOF.ASSETS] if x['id'] == action['asset']][0]
+        if asset['type'] == FOF.ASSET_RIGHTS:
+            return 0
         action['id'] = max([0] + [x['id'] for x in self._data[FOF.CORP_ACTIONS]]) + 1
-        action['asset'] = [action['asset'], action['asset']]
-        action['quantity'] = [-action['quantity'], 0.0]
-        action['cost_basis'] = 1.0
+        action['asset'] = action['asset']
+        action['quantity'] = -action['quantity']
+        action['outcome'] = []
         self.drop_extra_fields(action, ["value", "proceeds", "code", "asset_type", "jal_processed"])
         self._data[FOF.CORP_ACTIONS].append(action)
         return 1
 
+    def load_vestings(self, vestings):
+        cnt = 0
+        asset_payments_base = max([0] + [x['id'] for x in self._data[FOF.ASSET_PAYMENTS]]) + 1
+        for i, vesting in enumerate(vestings):
+            vesting['id'] = asset_payments_base + i
+            vesting['type'] = FOF.PAYMENT_STOCK_VESTING
+            self._data[FOF.ASSET_PAYMENTS].append(vesting)
+            cnt += 1
+        logging.info(self.tr("Stock vestings loaded: ") + f"{cnt} ({len(vestings)})")
+
     def load_cash_transactions(self, cash):
         cnt = 0
-
         dividends = list(filter(lambda tr: tr['type'] in ['Dividends', 'Payment In Lieu Of Dividends'], cash))
         asset_payments_base = max([0] + [x['id'] for x in self._data[FOF.ASSET_PAYMENTS]]) + 1
         for i, dividend in enumerate(dividends):
@@ -704,7 +743,6 @@ class StatementIBKR(StatementXML):
             self.drop_extra_fields(dividend, ["currency"])
             self._data[FOF.ASSET_PAYMENTS].append(dividend)
             cnt += 1
-
         asset_payments_base += cnt
         bond_interests = list(filter(lambda tr: tr['type'] in ['Bond Interest Paid', 'Bond Interest Received'], cash))
         for i, bond_interest in enumerate(bond_interests):
@@ -780,7 +818,7 @@ class StatementIBKR(StatementXML):
         parts = parts.groupdict()
         self.set_asset_counry(tax['asset'], parts['country'].lower())
         description = parts['description']
-        previous_tax = tax['amount'] if tax['amount'] >= 0 else 0
+        previous_tax = Decimal(tax['amount']) if Decimal(tax['amount']) >= Decimal('0') else Decimal('0')
         new_tax = -tax['amount'] if tax['amount'] < 0 else 0
 
         dividend = self.find_dividend4tax(tax['timestamp'], tax['account'], tax['asset'], previous_tax, new_tax, description)
@@ -808,25 +846,23 @@ class StatementIBKR(StatementXML):
                      (x['type'] == FOF.PAYMENT_DIVIDEND or x['type'] == FOF.PAYMENT_STOCK_DIVIDEND)
                      and x['asset'] == asset_id and x['account'] == account_id]
         account = [x for x in self._data[FOF.ACCOUNTS] if x["id"] == account_id][0]
-        currency = [x for x in self._data[FOF.ASSETS] if x["id"] == account['currency']][0]
-        currency_symbol = [x for x in self._data[FOF.SYMBOLS] if x["asset"] == currency['id']][0]
-        db_account = JalDB().get_account_id(account['number'], currency_symbol['symbol'])
+        db_account = JalAccount(data=account, search=True, create=False).id()
         asset = [x for x in self._data[FOF.ASSETS] if x["id"] == asset_id][0]
         isin = asset['isin'] if 'isin' in asset else ''
         symbols = [x for x in self._data[FOF.SYMBOLS] if x["asset"] == asset_id]
-        db_asset = JalDB().get_asset_id({'isin': isin, 'symbol': symbols[0]['symbol']})
-        if db_account is not None and db_asset is not None:
-            query = executeSQL(
-                "SELECT -id AS id, -account_id AS account, timestamp, number, "
-                "-asset_id AS asset, amount, tax, note as description FROM dividends "
-                "WHERE type=:div AND account_id=:account_id AND asset_id=:asset_id",
-                [(":div", Dividend.Dividend), (":account_id", db_account), (":asset_id", db_asset)],
-                forward_only=True)
-            while query.next():
-                db_dividend = readSQLrecord(query, named=True)
-                db_dividend['asset'] = asset_id
-                db_dividend['account'] = account_id
-                dividends.append(db_dividend)
+        db_asset = JalAsset(data={'isin': isin, 'symbol': symbols[0]['symbol']}, search=True, create=False).id()
+        if db_account and db_asset:
+            for db_dividend in Dividend.get_list(db_account, db_asset, Dividend.Dividend):
+                dividends.append({
+                    "id": -db_dividend.oid(),
+                    "account": account_id,
+                    "asset": asset_id,
+                    "timestamp": db_dividend.timestamp(),
+                    "number": db_dividend.number(),
+                    "amount": float(db_dividend.amount()),
+                    "tax": float(db_dividend.tax()),
+                    "description": db_dividend.note()
+                })
         if datetime.utcfromtimestamp(timestamp).timetuple().tm_yday < 75:
             # We may have wrong date in taxes before March, 15 due to tax correction
             range_start, _range_end = ManipulateDate.PreviousYear(day=datetime.utcfromtimestamp(timestamp))
@@ -834,7 +870,7 @@ class StatementIBKR(StatementXML):
         else:
             # For any other day - use exact time match
             dividends = [x for x in dividends if x['timestamp'] == timestamp]
-        dividends = [x for x in dividends if 'tax' not in x or (abs(x['tax'] - prev_tax) < 0.0001)]
+        dividends = [x for x in dividends if 'tax' not in x or (abs(Decimal(x['tax']) - prev_tax) < 0.0001)]
         dividends = sorted(dividends, key=lambda x: x['timestamp'])
 
         # Choose either Dividends or Payments in liue with regards to note of the matching tax
@@ -892,3 +928,10 @@ class StatementIBKR(StatementXML):
         if len(dividends) == 1:
             return dividends[0]
         return None
+
+    # Removes data that was used during XML processing but isn't needed in final output:
+    # Drop any assets with type 'right' as JAL won't import them
+    def strip_unused_data(self):
+        rights_id = [x['id'] for x in self._data[FOF.ASSETS] if x['type'] == FOF.ASSET_RIGHTS]
+        for asset_id in rights_id:
+            self.remove_asset(asset_id)
