@@ -1,9 +1,11 @@
 import logging
 import re
+import difflib
 from datetime import datetime
 
 from PySide6.QtWidgets import QApplication
-from jal.constants import Setup
+from jal.constants import Setup, PredefinedCategory
+from jal.db.settings import JalSettings
 from jal.data_import.statement import FOF, Statement_ImportError
 from jal.data_import.statement_xml import StatementXML
 from jal.net.downloader import QuoteDownloader
@@ -35,6 +37,10 @@ class OpenBroker_Asset:
     pass
 
 
+class OpenBroker_BrokerAsset:
+    pass
+
+
 # ----------------------------------------------------------------------------------------------------------------------
 class OpenBroker_Currency:
     pass
@@ -59,6 +65,7 @@ class OpenBroker_Exchange:
 class StatementOpenBroker(StatementXML):
     statements_path = '.'
     statement_tag = 'broker_report'
+    level_tag = 'asset_type_id'
 
     def __init__(self):
         super().__init__()
@@ -71,6 +78,7 @@ class StatementOpenBroker(StatementXML):
         open_loaders = {
             OpenBroker_AssetType: self.attr_asset_type,
             OpenBroker_Asset: self.attr_asset,
+            OpenBroker_BrokerAsset: self.attr_broker_asset,
             OpenBroker_Currency: self.attr_currency,
             OpenBroker_Exchange: self.attr_exchange
         }
@@ -101,11 +109,11 @@ class StatementOpenBroker(StatementXML):
             'spot_assets':    # Section describes end balances of account
                 {
                     'tag': 'item',
-                    'level': '',
+                    'level': '2',   # Take only money
                     'values': [('asset_name', 'asset', OpenBroker_Asset, None),
                                ('opening_position_plan', 'cash_begin', float, 0),
-                               ('closing_position_plan', 'cash_end', float, 0),
-                               ('closing_position_fact', 'cash_end_settled', float, 0)],
+                               ('closing_position_plan', 'cash_end_settled', float, 0),
+                               ('closing_position_fact', 'cash_end', float, 0)],
                     'loader': self.load_balances
                 },
             'spot_main_deals_conclusion':    # Deals list
@@ -130,7 +138,7 @@ class StatementOpenBroker(StatementXML):
                     'tag': 'item',
                     'level': '',
                     'values': [('operation_date', 'timestamp', datetime, None),
-                               ('security_name', 'asset', OpenBroker_Asset, None),
+                               ('security_name', 'asset', OpenBroker_BrokerAsset, None),
                                ('quantity', 'quantity', float, None),
                                ('comment', 'description', str, '')],
                     'loader': self.load_asset_operations
@@ -144,6 +152,19 @@ class StatementOpenBroker(StatementXML):
                                ('amount', 'amount', float, None),
                                ('comment', 'description', str, '')],
                     'loader': self.load_cash_operations
+                },
+            'spot_loan_deals_executed':
+                {
+                    'tag': 'item',
+                    'level': '',
+                    'values': [('operation_date', 'timestamp', datetime, None),
+                               ('operation_number', 'number', str, ''),
+                               ('instrument', 'ticker', str, None),
+                               ('loan_return_qty', 'qty', float, None),
+                               ('currency', 'currency', OpenBroker_Currency, None),
+                               ('profit', 'profit', float, None),
+                               ('brokers_fee', 'fee', float, None)],
+                    'loader': self.load_loans
                 }
         }
 
@@ -180,9 +201,29 @@ class StatementOpenBroker(StatementXML):
         asset_id = self.asset_id(asset_data)
         return asset_id
 
+    def attr_broker_asset(self, xml_element, attr_name, default_value):
+        if attr_name not in xml_element.attrib:
+            return default_value
+        broker_name = xml_element.attrib[attr_name].strip()
+        match = [x for x in self._data[FOF.SYMBOLS] if x['symbol'] == broker_name and "broker_symbol" in x]
+        if match:
+            if len(match) == 1:
+                return match[0]['asset']
+            else:
+                raise Statement_ImportError(self.tr("Multiple match for broker symbol: ") + f"'{broker_name}'")
+        # if not found via broker symbols then search through normal symbols
+        match = [x for x in self._data[FOF.SYMBOLS] if x['symbol'] == broker_name]
+        if match:
+            if len(match) == 1:
+                return match[0]['asset']
+            else:
+                raise Statement_ImportError(self.tr("Multiple match for symbol: ") + f"'{broker_name}'")
+
     # convert 'attr_name' currency code value into asset_id
     def attr_currency(self, xml_element, attr_name, default_value):
         if attr_name not in xml_element.attrib:
+            if xml_element.getparent().tag == 'spot_portfolio_security_params':
+                return self.currency_id("RUB")  # Currency might be missed for ADRs for example - let it be RUB
             return default_value
         currency_id = self.currency_id(xml_element.attrib[attr_name])
         if currency_id is None:
@@ -216,6 +257,7 @@ class StatementOpenBroker(StatementXML):
     def load_assets(self, assets):
         cnt = 0
         for asset in assets:
+            broker_symbol = asset['name'] if 'name' in asset else ''
             if asset['type'] == OpenBroker_AssetType.NotSupported:   # Skip not supported type of asset
                 continue
             if asset['exchange'] == "MOEX":
@@ -227,7 +269,13 @@ class StatementOpenBroker(StatementXML):
             if asset['exchange'] != '':  # don't store empty exchange
                 asset['note'] = asset['exchange']
             asset.pop('exchange')
-            self.asset_id(asset)
+            asset_id = self.asset_id(asset)
+            if broker_symbol:
+                if not [x['id'] for x in self._data[FOF.SYMBOLS] if x['symbol'] == broker_symbol]:
+                    symbol_id = max([0] + [x['id'] for x in self._data[FOF.SYMBOLS]]) + 1
+                    symbol = {"id": symbol_id, "asset": asset_id, "symbol": broker_symbol,
+                              "currency": asset['currency'], "broker_symbol": True}
+                    self._data[FOF.SYMBOLS].append(symbol)
             cnt += 1
         logging.info(self.tr("Securities loaded: ") + f"{cnt} ({len(assets)})")
 
@@ -260,7 +308,7 @@ class StatementOpenBroker(StatementXML):
             else:
                 trade['quantity'] = -trade['quantity_sell']
             amount = trade['proceeds'] + trade['accrued_interest']
-            if abs(abs(trade['price'] * trade['quantity']) - amount) >= Setup.DISP_TOLERANCE:
+            if abs(abs(trade['price'] * trade['quantity']) - amount) >= self.RU_PRICE_TOLERANCE:
                 trade['price'] = abs(amount / trade['quantity'])
             if abs(trade['accrued_interest']) > 0:
                 new_id = max([0] + [x['id'] for x in self._data[FOF.ASSET_PAYMENTS]]) + 1
@@ -274,38 +322,60 @@ class StatementOpenBroker(StatementXML):
 
     # this method loads only asset cancellations and puts it in self.asset_withdrawal for use in 'load_cash_operations'
     def load_asset_operations(self, asset_operations):
+        for operation in asset_operations:
+            if "Снятие ЦБ с учета. Погашение облигаций" in operation['description']:
+                self.load_bond_repayment(operation)
+            elif "Перевод ЦБ в" in operation['description']:
+                self.load_asset_transfer_out(operation)
+            else:
+                raise Statement_ImportError(self.tr("Unknown non-trade operation: ") + operation['description'])
+
+    def load_bond_repayment(self, operation):
         # Asset name is stored as alternative symbol and in self.asset_withdrawal[]
         bond_repayment_pattern = r"^.*Снятие ЦБ с учета\. Погашение облигаций - (?P<asset_name>.*)$"
-        for operation in asset_operations:
-            if "Снятие ЦБ с учета. Погашение облигаций" not in operation['description']:
-                raise Statement_ImportError(self.tr("Unknown non-trade operation: ")
-                                            + operation['description'])
-            parts = re.match(bond_repayment_pattern, operation['description'], re.IGNORECASE)
-            if parts is None:
-                raise Statement_ImportError(
-                    self.tr("Can't parse bond repayment description ") + f"'{operation['description']}'")
-            repayment_note = parts.groupdict()
-            if len(repayment_note) != bond_repayment_pattern.count("(?P<"):  # check expected number of matches
-                raise Statement_ImportError(
-                    self.tr("Can't detect bond name from description ") + f"'{repayment_note}'")
-            ticker = self._find_in_list(self._data[FOF.SYMBOLS], 'asset', operation['asset'])
-            if ticker['symbol'] != repayment_note['asset_name']:  # Store alternative depositary name
-                ticker = ticker.copy()
-                ticker['id'] = max([0] + [x['id'] for x in self._data[FOF.SYMBOLS]]) + 1
-                ticker['symbol'] = repayment_note['asset_name']
-                self._data[FOF.SYMBOLS].append(ticker)
-            new_id = max([0] + [x['id'] for x in self.asset_withdrawal]) + 1
-            record = {"id": new_id, "timestamp": operation['timestamp'], "asset": operation['asset'],
-                      "symbol": ticker['symbol'], "quantity": operation['quantity'], "note": operation['description']}
-            self.asset_withdrawal.append(record)
+        parts = re.match(bond_repayment_pattern, operation['description'], re.IGNORECASE)
+        if parts is None:
+            raise Statement_ImportError(
+                self.tr("Can't parse bond repayment description ") + f"'{operation['description']}'")
+        repayment_note = parts.groupdict()
+        if len(repayment_note) != bond_repayment_pattern.count("(?P<"):  # check expected number of matches
+            raise Statement_ImportError(
+                self.tr("Can't detect bond name from description ") + f"'{repayment_note}'")
+        ticker = self._find_in_list(self._data[FOF.SYMBOLS], 'asset', operation['asset'])
+        if ticker['symbol'] != repayment_note['asset_name']:  # Store alternative depositary name
+            ticker = ticker.copy()
+            ticker['id'] = max([0] + [x['id'] for x in self._data[FOF.SYMBOLS]]) + 1
+            ticker['symbol'] = repayment_note['asset_name']
+            ticker['broker_symbol'] = True
+            self._data[FOF.SYMBOLS].append(ticker)
+        new_id = max([0] + [x['id'] for x in self.asset_withdrawal]) + 1
+        record = {"id": new_id, "timestamp": operation['timestamp'], "asset": operation['asset'],
+                  "symbol": ticker['symbol'], "quantity": operation['quantity'], "note": operation['description']}
+        self.asset_withdrawal.append(record)
+
+    def load_asset_transfer_out(self, transfer):
+        transfer['id'] = max([0] + [x['id'] for x in self._data[FOF.TRANSFERS]]) + 1
+        transfer['account'] = [JalSettings().getValue('BaseCurrency'), 0, 0]
+        transfer['asset'] = [transfer['asset'], transfer['asset']]
+        transfer['withdrawal'] = transfer['deposit'] = -transfer['quantity']   # Withdrawal quantity is negative
+        transfer['fee'] = 0.0
+        self.drop_extra_fields(transfer, ["quantity"])
+        self._data[FOF.TRANSFERS].append(transfer)
 
     def load_cash_operations(self, cash_operations):
         cnt = 0
         operations = {
-            'Комиссия Брокера / ': None,              # These operations are included of trade's data
+            'Комиссия Брокера / ': None,              # These data are duplicated in normal trade data
+            'Комиссия Брокера за': self.cash_fee,
+            'Удержан налог на доход': None,           # Tax information is included into dividend payment data
             'Удержан налог на купонный доход': None,  # Tax information is included into interest payment data
             'Поставлены на торги средства клиента': self.transfer_in,
-            'Выплата дохода': self.asset_payment
+            'Списаны средства клиента': self.transfer_out,
+            'Выплата дохода': self.asset_payment,
+            'Возврат излишне удержанного налога': self.tax_refund,
+            'Проценты по предоставленным займам ЦБ': None,   # Loan payments are loaded in self.load_loans
+            'Удержан налог на прочий доход': self.cash_tax,
+            'Плата за остаток на счете': self.cash_interest
         }
 
         for cash in cash_operations:
@@ -331,6 +401,14 @@ class StatementOpenBroker(StatementXML):
                     "description": description}
         self._data[FOF.TRANSFERS].append(transfer)
 
+    def transfer_out(self, timestamp, account_id, amount, description):
+        account = [x for x in self._data[FOF.ACCOUNTS] if x["id"] == account_id][0]
+        new_id = max([0] + [x['id'] for x in self._data[FOF.TRANSFERS]]) + 1
+        transfer = {"id": new_id, "account": [account_id, 0, 0], "asset": [account['currency'], account['currency']],
+                    "timestamp": timestamp, "withdrawal": -amount, "deposit": -amount, "fee": 0.0,
+                    "description": description}
+        self._data[FOF.TRANSFERS].append(transfer)
+
     def asset_payment(self, timestamp, account_id, amount, description):
         payment_operations = {
             'НКД': self.interest_payment,
@@ -339,11 +417,49 @@ class StatementOpenBroker(StatementXML):
         payment_pattern = r"^.*\((?P<type>\w+).*\).*$"
         parts = re.match(payment_pattern, description, re.IGNORECASE)
         if parts is None:
-            raise Statement_ImportError(self.tr("Unknown payment description: ") + f"'{description}'")
-        try:
-            payment_operations[parts.groupdict()['type']](timestamp, account_id, amount, description)
-        except KeyError:
-            raise Statement_ImportError(self.tr("Unknown payment type: ") + f"'{parts.groupdict()['type']}'")
+            payment_pattern = r"^Выплата дохода клиент (?P<type>\d+) дивиденды (?P<asset>.*) налог к удержанию (?P<tax>\d+\.\d+) рублей$"
+            parts = re.match(payment_pattern, description, re.IGNORECASE)
+            if parts is None:
+                raise Statement_ImportError(self.tr("Unknown payment description: ") + f"'{description}'")
+            dividend_data = parts.groupdict()
+            asset_id = self.find_most_probable_asset(dividend_data['asset'])
+            tax = float(dividend_data['tax'])
+            self.dividend(timestamp, account_id, asset_id, amount, tax, description)
+        else:
+            try:
+                payment_operations[parts.groupdict()['type']](timestamp, account_id, amount, description)
+            except KeyError:
+                raise Statement_ImportError(self.tr("Unknown payment type: ") + f"'{parts.groupdict()['type']}'")
+
+    def tax_refund(self, timestamp, account_id, amount, description):
+        new_id = max([0] + [x['id'] for x in self._data[FOF.INCOME_SPENDING]]) + 1
+        payment = {'id': new_id, 'account': account_id, 'timestamp': timestamp, 'peer': 0,
+                   'lines': [{'amount': amount, 'category': -PredefinedCategory.Taxes, 'description': description}]}
+        self._data[FOF.INCOME_SPENDING].append(payment)
+
+    def cash_fee(self, timestamp, account_id, amount, description):
+        new_id = max([0] + [x['id'] for x in self._data[FOF.INCOME_SPENDING]]) + 1
+        payment = {'id': new_id, 'account': account_id, 'timestamp': timestamp, 'peer': 0,
+                   'lines': [{'amount': amount, 'category': -PredefinedCategory.Fees, 'description': description}]}
+        self._data[FOF.INCOME_SPENDING].append(payment)
+
+    def cash_tax(self, timestamp, account_id, amount, description):
+        new_id = max([0] + [x['id'] for x in self._data[FOF.INCOME_SPENDING]]) + 1
+        payment = {'id': new_id, 'account': account_id, 'timestamp': timestamp, 'peer': 0,
+                   'lines': [{'amount': amount, 'category': -PredefinedCategory.Taxes, 'description': description}]}
+        self._data[FOF.INCOME_SPENDING].append(payment)
+
+    def cash_interest(self, timestamp, account_id, amount, description):
+        new_id = max([0] + [x['id'] for x in self._data[FOF.INCOME_SPENDING]]) + 1
+        payment = {'id': new_id, 'account': account_id, 'timestamp': timestamp, 'peer': 0,
+                   'lines': [{'amount': amount, 'category': -PredefinedCategory.Interest, 'description': description}]}
+        self._data[FOF.INCOME_SPENDING].append(payment)
+
+    def dividend(self, timestamp, account_id, asset_id, amount, tax, description):
+        new_id = max([0] + [x['id'] for x in self._data[FOF.ASSET_PAYMENTS]]) + 1
+        payment = {"id": new_id, "type": FOF.PAYMENT_DIVIDEND, "account": account_id, "timestamp": timestamp,
+                   "asset": asset_id, "amount": amount, "tax": tax, "description": description}
+        self._data[FOF.ASSET_PAYMENTS].append(payment)
 
     def interest_payment(self, timestamp, account_id, amount, description):
         intrest_pattern = r"^Выплата дохода клиент (?P<account>\w+) \((?P<type>\w+) (?P<number>\d+) (?P<symbol>.*)\) налог.* (?P<tax>\d+\.\d+) рубл.*$"
@@ -389,3 +505,31 @@ class StatementOpenBroker(StatementXML):
                  "asset": asset_cancel['asset'], "quantity": qty, "price": price, "fee": 0.0,
                  "note": asset_cancel['note']}
         self._data[FOF.TRADES].append(trade)
+
+    def load_loans(self, loans):
+        for loan in loans:
+            new_id = max([0] + [x['id'] for x in self._data[FOF.INCOME_SPENDING]]) + 1
+            account_id = self.account_by_currency(loan['currency'])
+            note = f"Доход по сделке займа #{loan['number']}: {loan['qty']} x {loan['ticker']}"
+            fee_note = f"Комиссия за сделку займа #{loan['number']}: {loan['qty']} x {loan['ticker']}"
+            payment = {'id': new_id, 'account': account_id, 'timestamp': loan['timestamp'], 'peer': 0,
+                       'lines': [
+                           {'amount': loan['profit'], 'category': -PredefinedCategory.Interest, 'description': note},
+                           {'amount': -loan['fee'], 'category': -PredefinedCategory.Fees, 'description': fee_note}
+                       ]}
+            self._data[FOF.INCOME_SPENDING].append(payment)
+
+    # Removes data that was used during XML processing but isn't needed in final output:
+    # Drop any symbols with type "broker_symbol" as they are auxiliary for statement import only
+    def strip_unused_data(self):
+        self._data[FOF.SYMBOLS] = [x for x in self._data[FOF.SYMBOLS] if "broker_symbol" not in x]
+
+    # Broker report contains vague asset names in dividends
+    # This method tries to locate the best match in available assets
+    def find_most_probable_asset(self, asset_name):
+        match = difflib.get_close_matches(asset_name, [x['symbol'] for x in self._data[FOF.SYMBOLS]], 1)
+        if match:
+            asset = [x for x in self._data[FOF.SYMBOLS] if x['symbol'] == match[0]]
+            return asset[0]['asset']
+        else:
+            raise Statement_ImportError(self.tr("Can't match asset for ") + f"'{asset_name}'")
