@@ -5,15 +5,17 @@ import sys
 import os
 import logging
 from datetime import datetime, timezone
+from decimal import Decimal
 from collections import defaultdict
 
 from PySide6.QtCore import QObject
 from PySide6.QtWidgets import QDialog, QMessageBox
-from jal.constants import Setup, MarketDataFeed, PredefinedAsset
+from jal.constants import Setup, MarketDataFeed, PredefinedAsset, PredefindedAccountType
+from jal.db.helpers import get_app_path
+from jal.db.account import JalAccount
+from jal.db.asset import JalAsset
 from jal.db.settings import JalSettings
-from jal.db.helpers import account_last_date, get_app_path
-from jal.db.db import JalDB
-from jal.db.operations import Dividend, CorporateAction
+from jal.db.operations import LedgerTransaction, Dividend, CorporateAction
 from jal.widgets.account_select import SelectAccountDialog
 from jal.net.downloader import QuoteDownloader
 
@@ -38,6 +40,8 @@ class FOF:
     ASSET_FUTURES = "futures"
     ASSET_OPTION = "option"
     ASSET_WARRANT = "warrant"
+    ASSET_RIGHTS = "right"
+    ASSET_CRYPTO = "crypto"
 
     ACTION_MERGER = "merger"
     ACTION_SPLIT = "split"
@@ -45,10 +49,12 @@ class FOF:
     ACTION_SYMBOL_CHANGE = "symbol_change"
     ACTION_BOND_MATURITY = "bond_maturity"    # Isn't used in reality as will be put as ordinary sell operation
     ACTION_DELISTING = "delisting"
+    ACTION_RIGHTS_ISSUE = "rights_issue"      # Not in use as doesn't create any financial consequences
 
     PAYMENT_DIVIDEND = "dividend"
     PAYMENT_INTEREST = "interest"
     PAYMENT_STOCK_DIVIDEND = "stock_dividend"
+    PAYMENT_STOCK_VESTING = 'stock_vesting'
 
     def __init__(self):
         pass
@@ -70,6 +76,8 @@ class Statement_ImportError(Exception):
 
 # -----------------------------------------------------------------------------------------------------------------------
 class Statement(QObject):   # derived from QObject to have proper string translation
+    RU_PRICE_TOLERANCE = 1e-4   # TODO Probably need to switch imports to Decimal and remove it
+
     _asset_types = {
         FOF.ASSET_MONEY: PredefinedAsset.Money,
         FOF.ASSET_STOCK: PredefinedAsset.Stock,
@@ -79,6 +87,7 @@ class Statement(QObject):   # derived from QObject to have proper string transla
         FOF.ASSET_FUTURES: PredefinedAsset.Derivative,
         FOF.ASSET_OPTION: PredefinedAsset.Derivative,
         FOF.ASSET_WARRANT: PredefinedAsset.Derivative,
+        FOF.ASSET_CRYPTO: PredefinedAsset.Crypto
     }
     _corp_actions = {
         FOF.ACTION_MERGER: CorporateAction.Merger,
@@ -131,7 +140,7 @@ class Statement(QObject):   # derived from QObject to have proper string transla
     def load(self, filename: str) -> None:
         self._data = {}
         try:
-            with open(filename, 'r') as exchange_file:
+            with open(filename, 'r', encoding='utf-8') as exchange_file:
                 try:
                     self._data = json.load(exchange_file)
                 except json.JSONDecodeError:
@@ -158,8 +167,9 @@ class Statement(QObject):   # derived from QObject to have proper string transla
             if asset['type'] != FOF.ASSET_MONEY:
                 continue
             symbol = self._find_in_list(self._data[FOF.SYMBOLS], "asset", asset['id'])
-            asset_id = JalDB().get_asset_id({'symbol': symbol['symbol'], 'type': self._asset_types[asset['type']]})
-            if asset_id is not None:
+            asset_id = JalAsset(data={'symbol': symbol['symbol'], 'type': self._asset_types[asset['type']]},
+                                search=True, create=False).id()
+            if asset_id:
                 symbol['asset'] = -asset_id
                 old_id, asset['id'] = asset['id'], -asset_id
                 self._update_id("currency", old_id, asset_id)
@@ -171,8 +181,8 @@ class Statement(QObject):   # derived from QObject to have proper string transla
             if asset['id'] < 0:  # already matched
                 continue
             if 'isin' in asset:
-                asset_id = JalDB().get_asset_id({'isin': asset['isin']})
-                if asset_id is not None:
+                asset_id = JalAsset(data={'isin': asset['isin']}, search=True, create=False).id()
+                if asset_id:
                     old_id, asset['id'] = asset['id'], -asset_id
                     self._update_id("asset", old_id, asset_id)
 
@@ -182,8 +192,8 @@ class Statement(QObject):   # derived from QObject to have proper string transla
             if asset['asset'] < 0:  # already matched
                 continue
             if 'reg_number' in asset:
-                asset_id = JalDB().get_asset_id({'reg_number': asset['reg_number']})
-                if asset_id is not None:
+                asset_id = JalAsset(data={'reg_number': asset['reg_number']}, search=True, create=False).id()
+                if asset_id:
                     asset = self._find_in_list(self._data[FOF.ASSETS], "id", asset['asset'])
                     old_id, asset['id'] = asset['id'], -asset_id
                     self._update_id("asset", old_id, asset_id)
@@ -198,15 +208,17 @@ class Statement(QObject):   # derived from QObject to have proper string transla
             data = self._find_in_list(self._data[FOF.ASSETS_DATA], "asset", symbol['asset'])
             if data is not None:
                 self._uppend_keys_from(search_data, data, ['expiry'])
-            asset_id = JalDB().get_asset_id(search_data)
-            if asset_id is not None:
+            asset_id = JalAsset(data=search_data, search=True, create=False).id()
+            if asset_id:
                 old_id, asset['id'] = asset['id'], -asset_id
                 self._update_id("asset", old_id, asset_id)
 
     # Check and replace IDs for Accounts
     def _match_account_ids(self):
         for account in self._data[FOF.ACCOUNTS]:
-            account_id = JalDB().find_account(account['number'], -account['currency'])
+            account_data = account.copy()
+            account_data['currency'] = -account['currency']
+            account_id = JalAccount(data=account_data, search=True, create=False).id()
             if account_id:
                 old_id, account['id'] = account['id'], -account_id
                 self._update_id("account", old_id, account_id)
@@ -216,13 +228,38 @@ class Statement(QObject):   # derived from QObject to have proper string transla
         mutable_sections = [FOF.ACCOUNTS, FOF.ASSETS, FOF.SYMBOLS, FOF.ASSETS_DATA, FOF.TRADES, FOF.TRANSFERS,
                             FOF.CORP_ACTIONS, FOF.ASSET_PAYMENTS, FOF.INCOME_SPENDING]
         for section in mutable_sections:
+            if section not in self._data:
+                continue
             for element in self._data[section]:
-                for tag in element:
-                    if tag == tag_name:
-                        if type(element[tag]) == list:
-                            element[tag] = [-new_value if x == old_value else x for x in element[tag]]
-                        else:
-                            element[tag] = -new_value if element[tag] == old_value else element[tag]
+                if self._key_match(element, tag_name, old_value):
+                    if type(element[tag_name]) == list:
+                        element[tag_name] = [-new_value if x == old_value else x for x in element[tag_name]]
+                    else:
+                        element[tag_name] = -new_value if element[tag_name] == old_value else element[tag_name]
+        for element in self._data[FOF.CORP_ACTIONS]:  # Corporate actions have 'outcome' subsection with assets
+            for item in element['outcome']:
+                if self._key_match(item, tag_name, old_value):
+                    item[tag_name] = -new_value if item[tag_name] == old_value else item[tag_name]
+
+    # Deletes element if it's 'tag_name' key matches 'value'
+    def _delete_with_id(self, tag_name, value):
+        mutable_sections = [FOF.ACCOUNTS, FOF.ASSETS, FOF.SYMBOLS, FOF.ASSETS_DATA, FOF.TRADES, FOF.TRANSFERS,
+                            FOF.CORP_ACTIONS, FOF.ASSET_PAYMENTS, FOF.INCOME_SPENDING]
+        for section in mutable_sections:
+            self._data[section] = [x for x in self._data[section] if not self._key_match(x, tag_name, value)]
+
+    # returns True if dictionary 'element' has 'key' that matches 'value' or is a list with 'value'
+    def _key_match(self, element, key, value):
+        for tag in element:
+            if tag == key:
+                if type(element[tag]) == list:
+                    for x in element[tag]:
+                        if x == value:
+                            return True
+                else:
+                    if element[tag] == value:
+                        return True
+        return False
 
     def validate_format(self):
         schema_name = get_app_path() + Setup.IMPORT_PATH + os.sep + Setup.IMPORT_SCHEMA_NAME
@@ -255,13 +292,13 @@ class Statement(QObject):   # derived from QObject to have proper string transla
 
     def _check_period(self, period):
         if len(period) != 2:
-            logging.warning(self.tr("Statement period is invalid"))
-        if not FOF.ACCOUNTS in self._data:
+            raise Statement_ImportError(self.tr("Statement period is invalid"))
+        if FOF.ACCOUNTS not in self._data:
             return
         accounts = self._data[FOF.ACCOUNTS]
         for account in accounts:
             if account['id'] < 0:  # Checks if report is after last transaction recorded for account.
-                if period[0] < account_last_date(-account['id']):
+                if period[0] < JalAccount(-account['id']).last_operation_date():
                     if QMessageBox().warning(None, self.tr("Confirmation"),
                                              self.tr("Statement period starts before last recorded operation for the account. Continue import?"),
                                              QMessageBox.Yes, QMessageBox.No) == QMessageBox.No:
@@ -269,18 +306,17 @@ class Statement(QObject):   # derived from QObject to have proper string transla
 
     def _import_assets(self, assets):
         for asset in assets:
-            isin = asset['isin'] if 'isin' in asset else ''
-            name = asset['name'] if 'name' in asset else ''
-            country_code = asset['country'] if 'country' in asset else ''
             if asset['id'] < 0:
-                JalDB().update_asset_data(-asset['id'], {'isin': isin, 'name': name, 'country': country_code})
+                JalAsset(-asset['id']).update_data(asset)
                 continue
-            asset_id = JalDB().add_asset(self._asset_types[asset['type']], name, isin, country_code=country_code)
-            if asset_id:
-                old_id, asset['id'] = asset['id'], -asset_id
-                self._update_id("asset", old_id, asset_id)
+            asset_data = asset.copy()
+            asset_data['type'] = self._asset_types[asset_data['type']]
+            new_asset = JalAsset(data=asset_data, search=False, create=True)
+            if new_asset.id():
+                old_id, asset['id'] = asset['id'], -new_asset.id()
+                self._update_id("asset", old_id, new_asset.id())
                 if asset['type'] == FOF.ASSET_MONEY:
-                    self._update_id("currency", old_id, asset_id)
+                    self._update_id("currency", old_id, new_asset.id())
             else:
                 raise Statement_ImportError(self.tr("Can't create asset: ") + f"{asset}")
 
@@ -299,13 +335,13 @@ class Statement(QObject):   # derived from QObject to have proper string transla
                     source = self._sources[symbol['note']]
                 except KeyError:
                     source = MarketDataFeed.NA
-            JalDB().add_symbol(-symbol['asset'], symbol['symbol'], -symbol['currency'], note, data_source=source)
+            JalAsset(-symbol['asset']).add_symbol(symbol['symbol'], -symbol['currency'], note, data_source=source)
 
     def _import_asset_data(self, data):
         for detail in data:
             if detail['asset'] > 0:
                 raise Statement_ImportError(self.tr("Asset data aren't linked to asset: ") + f"{detail}")
-            JalDB().update_asset_data(-detail['asset'], detail)
+            JalAsset(-detail['asset']).update_data(detail)
     
     def _import_accounts(self, accounts):
         for account in accounts:
@@ -313,10 +349,13 @@ class Statement(QObject):   # derived from QObject to have proper string transla
                 continue
             if account['currency'] > 0:
                 raise Statement_ImportError(self.tr("Unmatched currency for account: ") + f"{account}")
-            account_id = JalDB().add_account(account['number'], -account['currency'])
-            if account_id:
-                old_id, account['id'] = account['id'], -account_id
-                self._update_id("account", old_id, account_id)
+            account_data = account.copy()
+            account_data['type'] = PredefindedAccountType.Investment if 'type' not in account_data else account_data['type']
+            account_data['currency'] = -account_data['currency']  # all currencies are already in db
+            new_account = JalAccount(data=account_data, search=True, create=True)
+            if new_account.id():
+                old_id, account['id'] = account['id'], -new_account.id()
+                self._update_id("account", old_id, new_account.id())
             else:
                 raise Statement_ImportError(self.tr("Can't create account: ") + f"{account}")
     
@@ -324,17 +363,18 @@ class Statement(QObject):   # derived from QObject to have proper string transla
         for action in actions:
             if action['account'] > 0:
                 raise Statement_ImportError(self.tr("Unmatched account for income/spending: ") + f"{action}")
+            action['account_id'] = -action.pop('account')
             if action['peer'] > 0:
                 raise Statement_ImportError(self.tr("Unmatched peer for income/spending: ") + f"{action}")
-            peer = JalDB().get_account_bank(-action['account']) if action['peer'] == 0 else -action['peer']
-            if len(action['lines']) != 1:   # FIXME - need support for multilines here
-                raise Statement_ImportError(self.tr("Unsupported income/spending: ") + f"{action}")
-            amount = action['lines'][0]['amount']
-            category = -action['lines'][0]['category']
-            if category <= 0:
-                raise Statement_ImportError(self.tr("Unmatched category for income/spending: ") + f"{action}")
-            description = action['lines'][0]['description']
-            JalDB().add_cash_transaction(-action['account'], peer, action['timestamp'], amount, category, description)
+            action['peer_id'] = -action.pop('peer')
+            if action['peer_id'] == 0:
+                action['peer_id'] = JalAccount(action['account_id']).organization()
+            for line in action['lines']:
+                if line['category'] >= 0:
+                    raise Statement_ImportError(self.tr("Unmatched category for income/spending: ") + f"{action}")
+                line['category_id'] = -line.pop('category')
+                line['note'] = line.pop('description')
+            LedgerTransaction().create_new(LedgerTransaction.IncomeSpending, action)
     
     def _import_transfers(self, transfers):
         for transfer in transfers:
@@ -344,23 +384,26 @@ class Statement(QObject):   # derived from QObject to have proper string transla
             for asset in transfer['asset']:
                 if asset > 0:
                     raise Statement_ImportError(self.tr("Unmatched asset for transfer: ") + f"{transfer}")
+            asset_types = [JalAsset(-x).type() for x in transfer['asset']]
+            if asset_types[0] != asset_types[1]:
+                raise Statement_ImportError(self.tr("Impossible to convert asset type in transfer: ") + f"{transfer}")
             if transfer['account'][0] == 0 or transfer['account'][1] == 0:
                 text = ''
                 pair_account = 1
                 if transfer['account'][0] == 0:  # Deposit
                     text = self.tr("Deposit of ") + f"{transfer['deposit']:.2f} " + \
-                           f"{JalDB().get_asset_name(-transfer['asset'][1])} " + \
+                           f"{JalAsset(-transfer['asset'][1]).symbol()} " + \
                            f"@{datetime.utcfromtimestamp(transfer['timestamp']).strftime('%d.%m.%Y')}\n" + \
                            self.tr("Select account to withdraw from:")
                     pair_account = -transfer['account'][1]
                 if transfer['account'][1] == 0:  # Withdrawal
                     text = self.tr("Withdrawal of ") + f"{transfer['withdrawal']:.2f} " + \
-                           f"{JalDB().get_asset_name(-transfer['asset'][0])} " + \
+                           f"{JalAsset(-transfer['asset'][0]).symbol()} " + \
                            f"@{datetime.utcfromtimestamp(transfer['timestamp']).strftime('%d.%m.%Y')}\n" + \
                            self.tr("Select account to deposit to:")
                     pair_account = -transfer['account'][0]
                 try:
-                    chosen_account = self._previous_accounts[JalDB().get_account_currency(pair_account)]
+                    chosen_account = self._previous_accounts[JalAccount(pair_account).currency()]
                 except KeyError:
                     chosen_account = self.select_account(text, pair_account, self._last_selected_account)
                 if chosen_account == 0:
@@ -370,52 +413,75 @@ class Statement(QObject):   # derived from QObject to have proper string transla
                     transfer['account'][0] = -chosen_account
                 if transfer['account'][1] == 0:
                     transfer['account'][1] = -chosen_account
-
-            description = transfer['description'] if 'description' in transfer else ''
-            JalDB().add_transfer(transfer['timestamp'], -transfer['account'][0], transfer['withdrawal'],
-                                 -transfer['account'][1], transfer['deposit'],
-                                 -transfer['account'][2], transfer['fee'], description)
+            if asset_types[0] != PredefinedAsset.Money:
+                transfer['asset'] = -transfer['asset'][0]
+            else:
+                transfer.pop('asset')
+            if 'description' in transfer:
+                transfer['note'] = transfer.pop('description')
+            if 'number' in transfer:
+                transfer.pop('number')   # it isn't stored in Jal
+            transfer['withdrawal_timestamp'] = transfer['deposit_timestamp'] = transfer.pop('timestamp')
+            transfer['withdrawal_account'] = -transfer['account'][0]
+            transfer['deposit_account'] = -transfer['account'][1]
+            transfer['fee_account'] = -transfer['account'][2]
+            transfer.pop('account')
+            if abs(transfer['fee']) < 1e-10:  # FIXME  Need to refactor this module for decimal usage
+                transfer.pop('fee_account')
+                transfer.pop('fee')
+            LedgerTransaction().create_new(LedgerTransaction.Transfer, transfer)
 
     def _import_trades(self, trades):
         for trade in trades:
             if trade['account'] > 0:
                 raise Statement_ImportError(self.tr("Unmatched account for trade: ") + f"{trade}")
+            trade['account_id'] = -trade.pop('account')
             if trade['asset'] > 0:
                 raise Statement_ImportError(self.tr("Unmatched asset for trade: ") + f"{trade}")
-            note = trade['note'] if 'note' in trade else ''
+            trade['asset_id'] = -trade.pop('asset')
+            trade['qty'] = trade.pop('quantity')
             if 'cancelled' in trade and trade['cancelled']:
-                JalDB().del_trade(-trade['account'], -trade['asset'], trade['timestamp'], trade['settlement'],
-                                  trade['number'], trade['quantity'], trade['price'], trade['fee'])
+                del trade['cancelled']          # Remove extra data
+                trade['qty'] = -trade['qty']    # Change side as cancellation is an opposite operation
+                oid = LedgerTransaction.locate_operation(LedgerTransaction.Trade, trade)
+                if oid:
+                    LedgerTransaction.get_operation(LedgerTransaction.Trade, oid).delete()
                 continue
-            JalDB().add_trade(-trade['account'], -trade['asset'], trade['timestamp'], trade['settlement'],
-                              trade['number'], trade['quantity'], trade['price'], trade['fee'], note)
+            LedgerTransaction().create_new(LedgerTransaction.Trade, trade)
 
     def _import_asset_payments(self, payments):
         for payment in payments:
             if payment['account'] > 0:
                 raise Statement_ImportError(self.tr("Unmatched account for payment: ") + f"{payment}")
+            payment['account_id'] = -payment.pop('account')
             if payment['asset'] > 0:
                 raise Statement_ImportError(self.tr("Unmatched asset for payment: ") + f"{payment}")
-            tax = payment['tax'] if 'tax' in payment else 0
+            payment['asset_id'] = -payment.pop('asset')
+            payment['note'] = payment.pop('description')
+            if 'price' in payment:
+                JalAsset(payment['asset_id']).set_quotes(
+                    [{'timestamp': payment['timestamp'], 'quote': Decimal(str(payment.pop('price')))}],  # FIXME Here should be no conversion to str
+                    JalAccount(payment['account_id']).currency())
             if payment['type'] == FOF.PAYMENT_DIVIDEND:
                 if payment['id'] > 0:  # New dividend
-                    JalDB().add_dividend(Dividend.Dividend, payment['timestamp'], -payment['account'],
-                                         -payment['asset'], payment['amount'], payment['description'], tax=tax)
+                    payment['type'] = Dividend.Dividend
+                    LedgerTransaction().create_new(LedgerTransaction.Dividend, payment)
                 else:  # Dividend exists, only tax to be updated
-                    JalDB().update_dividend_tax(-payment['id'], payment['tax'])
+                    dividend = LedgerTransaction.get_operation(LedgerTransaction.Dividend, -payment['id'])
+                    dividend.update_tax(payment['tax'])
             elif payment['type'] == FOF.PAYMENT_INTEREST:
-                if 'number' not in payment:
-                    payment['number'] = ''
-                JalDB().add_dividend(Dividend.BondInterest, payment['timestamp'], -payment['account'],
-                                     -payment['asset'], payment['amount'], payment['description'], payment['number'],
-                                     tax=tax)
+                payment['type'] = Dividend.BondInterest
+                LedgerTransaction().create_new(LedgerTransaction.Dividend, payment)
             elif payment['type'] == FOF.PAYMENT_STOCK_DIVIDEND:
                 if payment['id'] > 0:  # New dividend
-                    JalDB().add_dividend(Dividend.StockDividend, payment['timestamp'], -payment['account'],
-                                         -payment['asset'], payment['amount'], payment['description'],
-                                         payment['number'], tax=tax, price=payment['price'])
+                    payment['type'] = Dividend.StockDividend
+                    LedgerTransaction().create_new(LedgerTransaction.Dividend, payment)
                 else:  # Dividend exists, only tax to be updated
-                    JalDB().update_dividend_tax(-payment['id'], payment['tax'])
+                    dividend = LedgerTransaction.get_operation(LedgerTransaction.Dividend, -payment['id'])
+                    dividend.update_tax(payment['tax'])
+            elif payment['type'] == FOF.PAYMENT_STOCK_VESTING:
+                payment['type'] = Dividend.StockVesting
+                LedgerTransaction().create_new(LedgerTransaction.Dividend, payment)
             else:
                 raise Statement_ImportError(self.tr("Unsupported payment type: ") + f"{payment}")
 
@@ -423,26 +489,23 @@ class Statement(QObject):   # derived from QObject to have proper string transla
         for action in actions:
             if action['account'] > 0:
                 raise Statement_ImportError(self.tr("Unmatched account for corporate action: ") + f"{action}")
-            if type(action['asset']) == list:
-                asset_old = -action['asset'][0]
-                asset_new = -action['asset'][1]
-            else:
-                asset_old = asset_new = -action['asset']
-            if asset_old < 0 or asset_new < 0:
+            action['account_id'] = -action.pop('account')
+            if action['asset'] > 0:
                 raise Statement_ImportError(self.tr("Unmatched asset for corporate action: ") + f"{action}")
-            if type(action['quantity']) == list:
-                qty_old = action['quantity'][0]
-                qty_new = action['quantity'][1]
-            else:
-                qty_old = -1
-                qty_new = action['quantity']
+            action['asset_id'] = -action.pop('asset')
+            action['qty'] = action.pop('quantity')
+            action['note'] = action.pop('description')
+            for item in action['outcome']:
+                if item['asset'] > 0:
+                    raise Statement_ImportError(self.tr("Unmatched asset for corporate action: ") + f"{action}")
+                item['asset_id'] = -item.pop('asset')
+                item['qty'] = item.pop('quantity')
+                item['value_share'] = item.pop('share')
             try:
-                action_type = self._corp_actions[action['type']]
+                action['type'] = self._corp_actions[action.pop('type')]
             except KeyError:
                 raise Statement_ImportError(self.tr("Unsupported corporate action: ") + f"{action}")
-            JalDB().add_corporate_action(-action['account'], action_type, action['timestamp'], action['number'],
-                                         asset_old, qty_old, asset_new, qty_new,
-                                         action['cost_basis'], action['description'])
+            LedgerTransaction().create_new(LedgerTransaction.CorporateAction, action)
 
     def select_account(self, text, account_id, recent_account_id=0):
         if "pytest" in sys.modules:
@@ -452,7 +515,7 @@ class Statement(QObject):   # derived from QObject to have proper string transla
             return 0
         else:
             if dialog.store_account:
-                self._previous_accounts[JalDB().get_account_currency(dialog.account_id)] = dialog.account_id
+                self._previous_accounts[JalAccount(dialog.account_id).currency()] = dialog.account_id
             return dialog.account_id
 
     # Returns asset dictionary by asset id
@@ -582,3 +645,8 @@ class Statement(QObject):   # derived from QObject to have proper string transla
             else:
                 return
         self._uppend_keys_from(asset_data, asset_info, ['reg_number', 'expiry'])
+
+    # Removes asset and all links to it from self._data
+    def remove_asset(self, asset_id):
+        self._delete_with_id("asset", asset_id)
+        self._data[FOF.ASSETS] = [x for x in self._data[FOF.ASSETS] if x['id'] != asset_id]
