@@ -1,12 +1,13 @@
-import decimal
 from datetime import datetime
+from decimal import Decimal
 
 from PySide6.QtCore import Qt, Slot, QAbstractItemModel, QDate, QModelIndex, QLocale
 from PySide6.QtGui import QBrush, QFont
 from PySide6.QtWidgets import QHeaderView
-from jal.constants import Setup, CustomColor, BookAccount, PredefindedAccountType, AssetData
-from jal.db.helpers import executeSQL, readSQLrecord
-from jal.db.db import JalDB
+from jal.constants import CustomColor, PredefindedAccountType
+from jal.db.helpers import localize_decimal
+from jal.db.account import JalAccount
+from jal.db.asset import JalAsset
 from jal.db.settings import JalSettings
 from jal.widgets.delegates import GridLinesDelegate
 
@@ -133,22 +134,22 @@ class HoldingsModel(QAbstractItemModel):
                 if data['asset_is_currency']:
                     decimal_places = 2
                 else:
-                    decimal_places = -decimal.Decimal(str(data['qty']).rstrip('0')).as_tuple().exponent
-                    decimal_places = 6 if decimal_places > 6 else decimal_places
-                return QLocale().toString(data['qty'], 'f', decimal_places)
+                    decimal_places = -data['qty'].as_tuple().exponent
+                    decimal_places = max(min(decimal_places, 6), 0)
+                return localize_decimal(Decimal(data['qty']), decimal_places)
             else:
                 return ''
         elif column == 3:
-            if data['qty'] != 0 and data['value_i'] != 0:
+            if data['qty'] != Decimal('0') and data['value_i'] != Decimal('0'):
                 return f"{(data['value_i'] / data['qty']):,.4f}"
             else:
                 return ''
         elif column == 4:
-            return f"{data['quote']:,.4f}" if data['quote'] and data['qty'] != 0 else ''
+            return f"{float(data['quote']):,.4f}" if data['quote'] and float(data['qty']) != 0 else ''
         elif column == 5:
             return f"{data['share']:,.2f}" if data['share'] else '-.--'
         elif column == 6:
-            return f"{100.0 * data['profit_rel']:,.2f}" if data['profit_rel'] else ''
+            return f"{Decimal('100') * data['profit_rel']:,.2f}" if data['profit_rel'] else ''
         elif column == 7:
             return f"{data['profit']:,.2f}" if data['profit'] else ''
         elif column == 8:
@@ -208,7 +209,7 @@ class HoldingsModel(QAbstractItemModel):
     def setCurrency(self, currency_id):
         if self._currency != currency_id:
             self._currency = currency_id
-            self._currency_name = JalDB().get_asset_name(currency_id)
+            self._currency_name = JalAsset(currency_id).symbol()
             self.calculateHoldings()
 
     @Slot()
@@ -228,74 +229,61 @@ class HoldingsModel(QAbstractItemModel):
 
     # Populate table 'holdings' with data calculated for given parameters of model: _currency, _date,
     def calculateHoldings(self):
-        query = executeSQL(
-            "WITH "
-            "_last_quotes AS (SELECT MAX(timestamp) AS timestamp, asset_id, currency_id, quote "
-            "FROM quotes WHERE timestamp <= :holdings_timestamp GROUP BY asset_id, currency_id), "
-            "_last_assets AS ("
-            "SELECT id, SUM(t_value) AS total_value "
-            "FROM "
-            "("
-            "SELECT a.id, SUM(l.amount) AS t_value "
-            "FROM ledger AS l "
-            "LEFT JOIN accounts AS a ON l.account_id = a.id "
-            "WHERE (l.book_account=:money_book OR l.book_account=:liabilities_book) "
-            "AND a.type_id = :investments AND l.timestamp <= :holdings_timestamp GROUP BY a.id "
-            "UNION ALL "
-            "SELECT a.id, SUM(l.amount*q.quote) AS t_value "
-            "FROM ledger AS l "
-            "LEFT JOIN accounts AS a ON l.account_id = a.id "
-            "LEFT JOIN _last_quotes AS q ON l.asset_id = q.asset_id AND q.currency_id = a.currency_id "
-            "WHERE l.book_account=:assets_book AND a.type_id = :investments AND l.timestamp <= :holdings_timestamp "
-            "GROUP BY a.id"
-            ") "
-            "GROUP BY id HAVING ABS(total_value) > :tolerance) "
-            "SELECT h.currency_id, c.symbol AS currency, h.account_id, h.account, h.asset_id, "
-            "h.currency_id=h.asset_id AS asset_is_currency, coalesce(a.symbol, c.symbol) AS asset, "
-            "coalesce(a.full_name, c.full_name) AS asset_name, ad.value AS expiry, h.qty, h.value AS value_i, "
-            "h.quote, h.quote_a, h.total "
-            "FROM ("
-            "SELECT a.currency_id, l.account_id, a.name AS account, l.asset_id, sum(l.amount) AS qty, "
-            "sum(l.value) AS value, q.quote, q.quote*r.quote/ra.quote AS quote_a, t.total_value AS total "
-            "FROM ledger AS l "
-            "LEFT JOIN accounts AS a ON l.account_id = a.id "
-            "LEFT JOIN _last_quotes AS q ON l.asset_id = q.asset_id AND q.currency_id = a.currency_id "
-            "LEFT JOIN _last_quotes AS r ON a.currency_id = r.asset_id AND r.currency_id = :base_currency "
-            "LEFT JOIN _last_quotes AS ra ON ra.asset_id = :currency AND ra.currency_id = :base_currency "
-            "LEFT JOIN _last_assets AS t ON l.account_id = t.id "
-            "WHERE a.type_id = :investments AND l.book_account = :assets_book AND l.timestamp <= :holdings_timestamp "
-            "GROUP BY l.account_id, l.asset_id "
-            "HAVING ABS(qty) > :tolerance "
-            "UNION ALL "
-            "SELECT a.currency_id, l.account_id, a.name AS account, l.asset_id, sum(l.amount) AS qty, "
-            "0 AS value, 1, r.quote/ra.quote AS quote_a, t.total_value AS total "
-            "FROM ledger AS l "
-            "LEFT JOIN accounts AS a ON l.account_id = a.id "
-            "LEFT JOIN _last_quotes AS r ON a.currency_id = r.asset_id AND r.currency_id = :base_currency "
-            "LEFT JOIN _last_quotes AS ra ON ra.asset_id = :currency AND ra.currency_id = :base_currency "
-            "LEFT JOIN _last_assets AS t ON l.account_id = t.id "
-            "WHERE (l.book_account=:money_book OR l.book_account=:liabilities_book) "
-            "AND a.type_id = :investments AND l.timestamp <= :holdings_timestamp "
-            "GROUP BY l.account_id, l.asset_id "
-            "HAVING ABS(qty) > :tolerance "
-            ") AS h "
-            "LEFT JOIN assets_ext AS c ON c.id=h.currency_id AND c.currency_id=:base_currency "
-            "LEFT JOIN assets_ext AS a ON a.id=h.asset_id AND a.currency_id=h.currency_id " 
-            "LEFT JOIN asset_data AS ad ON ad.asset_id=a.id AND ad.datatype=:expiry "
-            "ORDER BY currency, account, asset_is_currency, asset",
-            [(":currency", self._currency), (":money_book", BookAccount.Money),
-             (":assets_book", BookAccount.Assets), (":liabilities_book", BookAccount.Liabilities),
-             (":holdings_timestamp", self._date), (":investments", PredefindedAccountType.Investment),
-             (":tolerance", Setup.DISP_TOLERANCE), (":expiry", AssetData.ExpiryDate),
-             (":base_currency", JalSettings().getValue('BaseCurrency'))], forward_only=True)
-        # Load data from SQL to tree
+        holdings = []
+        accounts = JalAccount.get_all_accounts(account_type=PredefindedAccountType.Investment)
+        for account in accounts:
+            account_holdings = []
+            assets = account.assets_list(self._date)
+            # Calculate cross-rate between account currency and display currency
+            rate = JalAsset(account.currency()).quote(self._date, JalSettings().getValue('BaseCurrency'))[1] / \
+                   JalAsset(self._currency).quote(self._date, JalSettings().getValue('BaseCurrency'))[1]
+            for asset_data in assets:
+                asset = asset_data['asset']
+                record = {
+                    "currency_id": account.currency(),
+                    "currency": JalAsset(account.currency()).symbol(),
+                    "account_id": account.id(),
+                    "account": account.name(),
+                    "asset_id": asset.id(),
+                    "asset_is_currency": False,
+                    "asset": asset.symbol(currency=account.currency()),
+                    "asset_name": asset.name(),
+                    "expiry": asset.expiry(),
+                    "qty": asset_data['amount'],
+                    "value_i": asset_data['value'],
+                    "quote": asset.quote(self._date, account.currency())[1],  # [0] is a timestamp
+                    "quote_a": rate * asset.quote(self._date, account.currency())[1]
+                }
+                account_holdings.append(record)
+            money = account.get_asset_amount(self._date, account.currency())
+            if money:
+                account_holdings.append({
+                    "currency_id": account.currency(),
+                    "currency": JalAsset(account.currency()).symbol(),
+                    "account_id": account.id(),
+                    "account": account.name(),
+                    "asset_id": account.currency(),
+                    "asset_is_currency": True,
+                    "asset": JalAsset(account.currency()).symbol(),
+                    "asset_name": JalAsset(account.currency()).name(),
+                    "expiry": 0,
+                    "qty": money,
+                    "value_i": Decimal('0'),
+                    "quote": Decimal('1'),
+                    "quote_a": rate
+                })
+            account_total = sum(x['qty'] * x['quote'] for x in account_holdings)
+            for record in account_holdings:
+                record['total'] = account_total
+            holdings += account_holdings
+        holdings = sorted(holdings, key=lambda x: (x['currency'], x['account'], x['asset_is_currency'], x['asset']))
+
         self._root = TreeItem({})
         currency = 0
         c_node = None
         account = 0
         a_node = None
-        while query.next():
-            values = readSQLrecord(query, named=True)
+        for values in holdings:
             values['level'] = 2
             if values['currency_id'] != currency:
                 currency = values['currency_id']
@@ -303,7 +291,7 @@ class HoldingsModel(QAbstractItemModel):
                 c_node.data['level'] = 0
                 c_node.data['asset_name'] = ''
                 c_node.data['expiry'] = 0
-                c_node.data['qty'] = 0
+                c_node.data['qty'] = Decimal('0')
                 self._root.appendChild(c_node)
             if values['account_id'] != account:
                 account = values['account_id']
@@ -311,23 +299,24 @@ class HoldingsModel(QAbstractItemModel):
                 a_node.data['level'] = 1
                 a_node.data['asset_name'] = ''
                 a_node.data['expiry'] = 0
-                a_node.data['qty'] = 0
+                a_node.data['qty'] = Decimal('0')
                 c_node.appendChild(a_node)
             if values['quote']:
                 if values['asset_is_currency']:
-                    profit = 0
+                    profit = Decimal('0')
                 else:
                     profit = values['quote'] * values['qty'] - values['value_i']
-                if values['value_i'] != 0:
+                if values['value_i'] != Decimal('0'):
                     profit_relative = values['quote'] * values['qty'] / values['value_i'] - 1
                 else:
-                    profit_relative = 0
+                    profit_relative = Decimal('0')
                 value = values['quote'] * values['qty']
-                share = 100.0 * value / values['total']
-                value_adjusted = values['quote_a'] * values['qty'] if values['quote_a'] else 0
+                share = Decimal('100.0') * value / values['total']
+                value_adjusted = values['quote_a'] * values['qty'] if values['quote_a'] else Decimal('0')
                 values.update(dict(zip(self.calculated_names, [share, profit, profit_relative, value, value_adjusted])))
             else:
-                values.update(dict(zip(self.calculated_names, [0, 0, 0, 0, 0])))
+                values.update(dict(zip(self.calculated_names,
+                                       [Decimal('0'), Decimal('0'), Decimal('0'), Decimal('0'), Decimal('0')])))
             node = TreeItem(values, a_node)
             a_node.appendChild(node)
 
@@ -340,13 +329,12 @@ class HoldingsModel(QAbstractItemModel):
             for j in range(currency_child.count()):  # Calculate share of each account within currency
                 if currency_child.data['value']:
                     currency_child.getChild(j).data['share'] = \
-                        100.0 * currency_child.getChild(j).data['value'] / currency_child.data['value']
+                        Decimal('100') * currency_child.getChild(j).data['value'] / currency_child.data['value']
         # Get full total of totals for all currencies adjusted to common currency
         total = sum([self._root.getChild(i).data['value_a'] for i in range(self._root.count())])
         for i in range(self._root.count()):  # Calculate share of each currency (adjusted to common currency)
-            if total != 0:
-                self._root.getChild(i).data['share'] = 100.0 * self._root.getChild(i).data[
-                    'value_a'] / total
+            if total != Decimal('0'):
+                self._root.getChild(i).data['share'] = Decimal('100') * self._root.getChild(i).data['value_a'] / total
             else:
                 self._root.getChild(i).data['share'] = None
         self.modelReset.emit()
@@ -358,4 +346,4 @@ class HoldingsModel(QAbstractItemModel):
         value = sum([node.getChild(i).data['value'] for i in range(node.count())])
         value_adjusted = sum([node.getChild(i).data['value_a'] for i in range(node.count())])
         profit_relative = profit / (value - profit) if value != profit else 0
-        node.data.update(dict(zip(self.calculated_names, [0, profit, profit_relative, value, value_adjusted])))
+        node.data.update(dict(zip(self.calculated_names, [Decimal('0'), profit, profit_relative, value, value_adjusted])))
