@@ -1,21 +1,14 @@
-from datetime import datetime
+from decimal import Decimal
 from PySide6.QtCore import Qt, Slot, QAbstractTableModel, QDate
 from PySide6.QtGui import QBrush, QFont
 from PySide6.QtWidgets import QStyledItemDelegate, QHeaderView
 from jal.constants import CustomColor
-from jal.db.helpers import executeSQL, readSQLrecord
+from jal.db.ledger import Ledger
 from jal.db.operations import LedgerTransaction
+from jal.widgets.helpers import ts2str
 
 
 class OperationsModel(QAbstractTableModel):
-    _tables = {
-        LedgerTransaction.IncomeSpending: "actions",
-        LedgerTransaction.Dividend: "dividends",
-        LedgerTransaction.Trade: "trades",
-        LedgerTransaction.Transfer: "transfers",
-        LedgerTransaction.CorporateAction: "corp_actions"
-    }
-
     def __init__(self, parent_view):
         super().__init__(parent_view)
         self._columns = [" ", self.tr("Timestamp"), self.tr("Account"), self.tr("Notes"),
@@ -82,15 +75,15 @@ class OperationsModel(QAbstractTableModel):
         if column == 0:
             return operation.label()
         elif column == 1:
-            date_time = datetime.utcfromtimestamp(operation.timestamp()).strftime('%d/%m/%Y %H:%M:%S')
+            date_time = ts2str(operation.timestamp())
             if operation.number():
                 date_time += f"\n# {operation.number()}"
             return date_time
         elif column == 2:
-            if operation.asset():
-                return operation.account() + "\n" + operation.asset()
+            if operation.asset_name():
+                return operation.account_name() + "\n" + operation.asset_name()
             else:
-                return operation.account()
+                return operation.account_name()
         elif column == 3:
             return operation.description()
         elif column == 4:
@@ -100,7 +93,7 @@ class OperationsModel(QAbstractTableModel):
         elif column == 6:
             return operation.value_currency()
         else:
-            return True, "Unexpected column number"
+            assert False, "Unexpected column number"
 
     def configureView(self):
         self._view.setColumnWidth(0, 10)
@@ -157,20 +150,13 @@ class OperationsModel(QAbstractTableModel):
         if self._begin == 0 and self._end == 0:
             self._row_count = 0
         else:
-            query_text = f"SELECT * FROM operation_sequence WHERE timestamp>={self._begin} AND timestamp<={self._end}"
-            if self._account:
-                query_text += f" AND account_id={self._account}"
-            query = executeSQL(query_text, forward_only=True)
-            while query.next():
-                self._data.append(readSQLrecord(query, named=True))
+            self._data = Ledger.get_operations_sequence(self._begin, self._end, self._account)
         self.modelReset.emit()
 
     def deleteRows(self, rows):
         for row in rows:
             if (row >= 0) and (row < len(self._data)):
-                table_name = self._tables[self._data[row]['op_type']]
-                query = f"DELETE FROM {table_name} WHERE id={self._data[row]['id']}"
-                _ = executeSQL(query)
+                LedgerTransaction.get_operation(self._data[row]['op_type'], self._data[row]['id']).delete()
         self.prepareData()
 
 
@@ -183,18 +169,13 @@ class ColoredAmountsDelegate(QStyledItemDelegate):
     def paint(self, painter, option, index):
         painter.save()
         data = index.model().data(index)
-        if len(data) == 1:
-            self.draw_value(option.rect, painter, data[0])
-        elif len(data) == 2:
-            rect = option.rect
-            H = rect.height()
-            Y = rect.top()
-            rect.setHeight(H / 2)
-            self.draw_value(option.rect, painter, data[0])
-            rect.moveTop(Y + H / 2)
-            self.draw_value(option.rect, painter, data[1])
-        else:
-            assert False
+        rect = option.rect
+        H = rect.height()
+        Y = rect.top()
+        rect.setHeight(H / len(data))
+        for i, item in enumerate(data):
+            rect.moveTop(Y + i * (H / len(data)))
+            self.draw_value(option.rect, painter, item)
         painter.restore()
 
     def draw_value(self, rect, painter, value):
@@ -209,5 +190,8 @@ class ColoredAmountsDelegate(QStyledItemDelegate):
                     pen.setColor(CustomColor.DarkRed)
             painter.setPen(pen)
             painter.drawText(rect, Qt.AlignRight | Qt.AlignVCenter, f"{value:+,.2f}")
+            if abs(value - round(value, 2)) > Decimal('0'):  # Underline decimal part
+                shift = painter.fontMetrics().horizontalAdvance(f"{value:+,.2f}"[-2:])
+                painter.drawLine(rect.right() - shift, rect.bottom(), rect.right(), rect.bottom())
         except TypeError:
             pass
